local compost = CompostLib:GetInstance('compost-1')
local babble = BabbleLib:GetInstance("1.0")
local tablet = TabletLib:GetInstance('1.0')
local dewdrop = DewdropLib:GetInstance('1.0')

TopScoreFu = FuBarPlugin:GetInstance("1.2"):new({
	name          = TopScoreFuLocals.NAME,
	description   = TopScoreFuLocals.DESCRIPTION,
	version       = "1.1." .. string.sub("$Revision: 1551 $", 12, -3),
	releaseDate   = string.sub("$Date: 2006-05-06 23:08:11 -0500 (Sat, 06 May 2006) $", 8, 17),
	aceCompatible = 103,
	author        = "ckknight",
	email         = "ckknight@gmail.com",
	website       = "http://ckknight.wowinterface.com/",
	category      = "combat",
	db            = AceDatabase:new("TopScoreFuDB"),
	defaults      = {
		splash = true,
		noise = true,
		screenshot = false,
		includeHeals = true,
		includeDamage = true,
		onlyPvP = false,
		showTrivial = false,
		posX = 0,
		posY = 0,
		filters = {},
	},
	charDefaults  = {
		hits = {},
	},
	cmd           = AceChatCmd:new(TopScoreFuLocals.COMMANDS, TopScoreFuLocals.CMD_OPTIONS),
	loc           = TopScoreFuLocals,
	hasIcon       = true,
	canHideText   = true,
})

function TopScoreFu:SetPosition(text)
	local _,_,x,y = string.find(text, "(%-?%d+),? (%-?%d+)")
	x, y = tonumber(x), tonumber(y)
	if x == nil or y == nil then
		self.cmd:msg(self.loc.TEXT_SET_POSITION_ERROR)
	else
		self.data.posX = x
		self.data.posY = y
		TopScoreFuSplash:ClearAllPoints()
		TopScoreFuSplash:SetPoint("CENTER", UIParent, "CENTER", self.data.posX, self.data.posY)
		self.cmd:msg(self.loc.PATTERN_SET_POSITION, x, y)
	end
end

function TopScoreFu:ResetScores(loud)
	compost:Erase(self.charData.hits)
	if loud then
		self.cmd:msg(self.loc.TEXT_SCORES_RESET)
	end
	self:Update()
end

function TopScoreFu:IsShowingSplash()
	return self.data.splash
end

function TopScoreFu:ToggleShowingSplash(loud)
	self.data.splash = not self.data.splash
	if loud then
		self.cmd:status(self.loc.ARGUMENT_SHOWSPLASH, self.data.splash and 1 or 0, FuBarLocals.MAP_ONOFF)
	end
	return self.data.splash
end

function TopScoreFu:IsShowingTrivial()
	return self.data.showTrivial
end

function TopScoreFu:ToggleShowingTrivial(loud)
	self.data.showTrivial = not self.data.showTrivial
	if loud then
		self.cmd:status(self.loc.ARGUMENT_SHOWTRIVIAL, self.data.showTrivial and 1 or 0, FuBarLocals.MAP_ONOFF)
	end
	self:Update()
	return self.data.showTrivial
end

function TopScoreFu:IsPlayingNoise()
	return self.data.noise
end

function TopScoreFu:TogglePlayingNoise(loud)
	self.data.noise = not self.data.noise
	if loud then
		self.cmd:status(self.loc.ARGUMENT_PLAYNOISE, self.data.noise and 1 or 0, FuBarLocals.MAP_ONOFF)
	end
	return self.data.noise
end

function TopScoreFu:IsTakingScreenshots()
	return self.data.screenshot
end

function TopScoreFu:ToggleTakingScreenshots(loud)
	self.data.screenshot = not self.data.screenshot
	if loud then
		self.cmd:status(self.loc.ARGUMENT_SCREENSHOT, self.data.screenshot and 1 or 0, FuBarLocals.MAP_ONOFF)
	end
	return self.data.screenshot
end

function TopScoreFu:IsIncludingHeals()
	return self.data.includeHeals
end

function TopScoreFu:ToggleIncludingHeals(loud)
	self.data.includeHeals = not self.data.includeHeals
	if loud then
		self.cmd:status(self.loc.ARGUMENT_HEALING, self.data.includeHeals and 1 or 0, FuBarLocals.MAP_ONOFF)
	end
	self:Update()
	return self.data.includeHeals
end

function TopScoreFu:IsIncludingDamage()
	return self.data.includeDamage
end

function TopScoreFu:ToggleIncludingDamage(loud)
	self.data.includeDamage = not self.data.includeDamage
	if loud then
		self.cmd:status(self.loc.ARGUMENT_DAMAGE, self.data.includeDamage and 1 or 0, FuBarLocals.MAP_ONOFF)
	end
	self:Update()
	return self.data.includeDamage
end

function TopScoreFu:IsOnlyPvP()
	return self.data.onlyPvP
end

function TopScoreFu:ToggleOnlyPvP(loud)
	self.data.onlyPvP = not self.data.onlyPvP
	if loud then
		self.cmd:status(self.loc.ARGUMENT_ONLYPVP, self.data.onlyPvP and 1 or 0, FuBarLocals.MAP_ONOFF)
	end
	self:Update()
	return self.data.onlyPvP
end

function TopScoreFu:IsFiltering(spell)
	return self.data.filters[spell]
end

function TopScoreFu:ToggleFiltering(spell)
	self.data.filters[spell] = not self.data.filters[spell]
	self:Update()
	return self.data.filters[spell]
end

function TopScoreFu:Purge(spell)
	self.data.filters[spell] = nil
	self.charData.hits[spell] = nil
	if not next(self.charData.hits) then
		dewdrop:Close(2)
		dewdrop:Refresh(1)
	end
end

function TopScoreFu:Initialize()
	local frame = CreateFrame("MessageFrame", "TopScoreFuSplash", UIParent)
	frame:SetWidth(512)
	frame:SetHeight(100)
	frame:SetPoint("CENTER", UIParent, "CENTER")
	frame:SetFontObject(NumberFontNormalHuge)
	frame:SetFrameStrata("LOW")
	
	for spell, _ in self.charData.hits do
		if not self:DoesSpellExist(spell) then
			self.charData.hits[spell] = false
		end
	end
end

function TopScoreFu:Enable()
	self:RegisterEvent("CHAT_MSG_SPELL_SELF_BUFF", "OnChatMsgSpellSelfBuff")
	self:RegisterEvent("CHAT_MSG_COMBAT_SELF_HITS", "OnChatMsgCombatSelfHits")
	self:RegisterEvent("CHAT_MSG_SPELL_SELF_DAMAGE", "OnChatMsgSpellSelfDamage")
	self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_HOSTILEPLAYER_DAMAGE", "OnChatMsgSpellPeriodicHostileplayerDamage")
	self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE", "OnChatMsgSpellPeriodicCreatureDamage")
	self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS", "OnChatMsgSpellPeriodicSelfBuffs")
	TopScoreFuSplash:ClearAllPoints()
	TopScoreFuSplash:SetPoint("CENTER", UIParent, "CENTER", self.data.posX, self.data.posY)
end

function TopScoreFu:Report()
	local report = compost:Acquire(
		compost:AcquireHash(
			'text', self.loc.ARGUMENT_SHOWSPLASH,
			'val', self:IsShowingSplash() and 1 or 0,
			'map', FuBarLocals.MAP_ONOFF
		),
		compost:AcquireHash(
			'text', self.loc.ARGUMENT_PLAYNOISE,
			'val', self:IsPlayingNoise() and 1 or 0,
			'map', FuBarLocals.MAP_ONOFF
		),
		compost:AcquireHash(
			'text', self.loc.ARGUMENT_SCREENSHOT,
			'val', self:IsTakingScreenshots() and 1 or 0,
			'map', FuBarLocals.MAP_ONOFF
		),
		compost:AcquireHash(
			'text', self.loc.ARGUMENT_HEALING,
			'val', self:IsIncludingHeals() and 1 or 0,
			'map', FuBarLocals.MAP_ONOFF
		),
		compost:AcquireHash(
			'text', self.loc.ARGUMENT_ONLYPVP,
			'val', self:IsOnlyPvP() and 1 or 0,
			'map', FuBarLocals.MAP_ONOFF
		),
		compost:AcquireHash(
			'text', self.loc.ARGUMENT_SHOWTRIVIAL,
			'val', self:IsShowingTrivial() and 1 or 0,
			'map', FuBarLocals.MAP_ONOFF
		)
	)
	self.cmd:report(report)
	compost:Reclaim(report, 1)
end

function TopScoreFu:MenuSettings(level, value)
	if level == 1 then
		dewdrop:AddLine(
			'text', self.loc.MENU_SHOW_SPLASH,
			'arg1', self,
			'func', "ToggleShowingSplash",
			'checked', self:IsShowingSplash()
		)
		
		dewdrop:AddLine(
			'text', self.loc.MENU_PLAY_NOISE,
			'arg1', self,
			'func', "TogglePlayingNoise",
			'checked', self:IsPlayingNoise()
		)
		
		dewdrop:AddLine(
			'text', self.loc.MENU_TAKE_SCREENSHOTS,
			'arg1', self,
			'func', "ToggleTakingScreenshots",
			'checked', self:IsTakingScreenshots()
		)
		
		dewdrop:AddLine(
			'text', self.loc.MENU_RESET_SCORES,
			'arg1', self,
			'func', "ResetScores",
			'closeWhenClicked', true
		)
		
		dewdrop:AddLine()
		
		dewdrop:AddLine(
			'text', self.loc.MENU_FILTER,
			'hasArrow', true,
			'value', "filter"
		)
		
		dewdrop:AddLine(
			'text', self.loc.MENU_PURGE,
			'hasArrow', next(self.charData.hits),
			'value', "purge"
		)
	elseif level == 2 then
		if value == "filter" then
			dewdrop:AddLine(
				'text', self.loc.MENU_VS_MONSTERS,
				'arg1', self,
				'func', "ToggleOnlyPvP",
				'checked', not self:IsOnlyPvP()
			)
			
			dewdrop:AddLine(
				'text', self.loc.MENU_INCLUDE_HEALING,
				'arg1', self,
				'func', "ToggleIncludingHeals",
				'checked', self:IsIncludingHeals()
			)
			
			dewdrop:AddLine(
				'text', self.loc.MENU_INCLUDE_DAMAGE,
				'arg1', self,
				'func', "ToggleIncludingDamage",
				'checked', self:IsIncludingDamage()
			)
			
			dewdrop:AddLine(
				'text', self.loc.MENU_SHOW_TRIVIAL,
				'arg1', self,
				'func', "ToggleShowingTrivial",
				'checked', self:IsShowingTrivial()
			)
			
			dewdrop:AddLine()
			
			for spell, t in pairs(self.charData.hits) do
				dewdrop:AddLine(
					'text', spell,
					'arg1', self,
					'func', "ToggleFiltering",
					'arg2', spell,
					'checked', not self:IsFiltering(spell)
				)
			end
		elseif value == "purge" then
			for spell, t in pairs(self.charData.hits) do
				dewdrop:AddLine(
					'text', spell,
					'arg1', self,
					'func', "Purge",
					'arg2', spell
				)
			end
		end
	end
end

function TopScoreFu:OnChatMsgSpellSelfBuff()
	local spell, target, amount, isCrit
	
	target = "self"
	isCrit = true
	spell, amount = babble.Deformat(arg1, HEALEDCRITSELFSELF)
	if not spell then
		isCrit = false
		spell, amount = babble.Deformat(arg1, HEALEDSELFSELF)
		if not spell then
			isCrit = true
			spell, target, amount = babble.Deformat(arg1, HEALEDCRITSELFOTHER)
			if not spell then
				isCrit = false
				spell, target, amount = babble.Deformat(arg1, HEALEDSELFOTHER)
				if not spell then
					target = "self"
					isCrit = true
					spell, amount = babble.Deformat(arg1, HEALEDCRITSELFSELF, RESIST_TRAILER)
					if not spell then
						isCrit = false
						spell, amount = babble.Deformat(arg1, HEALEDSELFSELF, RESIST_TRAILER)
						if not spell then
							isCrit = true
							spell, target, amount = babble.Deformat(arg1, HEALEDCRITSELFOTHER, RESIST_TRAILER)
							if not spell then
								isCrit = false
								spell, target, amount = babble.Deformat(arg1, HEALEDSELFOTHER, RESIST_TRAILER)
							end
						end
					end
				end
			end
		end
	end
	if spell then
		self:RecordHit(spell, tonumber(amount), target, isCrit, true)
	end
end

function TopScoreFu:OnChatMsgCombatSelfHits()
	local target, amount, isCrit
	isCrit = true
	target, amount = babble.Deformat(arg1, COMBATHITCRITSCHOOLSELFOTHER)
	if not target then
		target, amount, _ = babble.Deformat(arg1, COMBATHITCRITSELFOTHER)
		if not target then
			isCrit = false
			target, amount = babble.Deformat(arg1, COMBATHITSCHOOLSELFOTHER)
			if not target then
				target, amount, _ = babble.Deformat(arg1, COMBATHITSELFOTHER)
				if not target then
					isCrit = true
					target, amount = babble.Deformat(arg1, COMBATHITCRITSCHOOLSELFOTHER, RESIST_TRAILER)
					if not target then
						target, amount, _ = babble.Deformat(arg1, COMBATHITCRITSELFOTHER, RESIST_TRAILER)
						if not target then
							isCrit = false
							target, amount = babble.Deformat(arg1, COMBATHITSCHOOLSELFOTHER, RESIST_TRAILER)
							if not target then
								target, amount, _ = babble.Deformat(arg1, COMBATHITSELFOTHER, RESIST_TRAILER)
							end
						end
					end
				end
			end
		end
	end
	if target then
		self:RecordHit(babble.GetLocalizedSpell("ATTACK"), tonumber(amount), target, isCrit, false)
	end
end

function TopScoreFu:OnChatMsgSpellSelfDamage()
	local spell, target, amount, isCrit
	isCrit = true
	spell, target, amount = babble.Deformat(arg1, SPELLLOGCRITSCHOOLSELFOTHER)
	if not spell then
		spell, target, amount = babble.Deformat(arg1, SPELLLOGCRITSELFOTHER)
		if not spell then
			isCrit = false
			spell, target, amount = babble.Deformat(arg1, SPELLLOGSCHOOLSELFOTHER)
			if not spell then
				spell, target, amount = babble.Deformat(arg1, SPELLLOGSELFOTHER)
				if not spell then
					isCrit = true
					spell, target, amount = babble.Deformat(arg1, SPELLLOGCRITSCHOOLSELFOTHER, RESIST_TRAILER)
					if not spell then
						spell, target, amount = babble.Deformat(arg1, SPELLLOGCRITSELFOTHER, RESIST_TRAILER)
						if not spell then
							isCrit = false
							spell, target, amount = babble.Deformat(arg1, SPELLLOGSCHOOLSELFOTHER, RESIST_TRAILER)
							if not spell then
								spell, target, amount = babble.Deformat(arg1, SPELLLOGSELFOTHER, RESIST_TRAILER)
							end
						end
					end
				end
			end
		end
	end
	if spell then
		self:RecordHit(spell, tonumber(amount), target, isCrit, false)
	end
end

function TopScoreFu:OnChatMsgSpellPeriodicHostileplayerDamage()
	local target, amount, _, spell = babble.Deformat(arg1, PERIODICAURADAMAGESELFOTHER)
	if not target then
		target, amount, _, spell = babble.Deformat(arg1, PERIODICAURADAMAGESELFOTHER, RESIST_TRAILER)
	end
	if target then
		self:RecordHit(spell, tonumber(amount), target, false, false)
	end
end

function TopScoreFu:OnChatMsgSpellPeriodicCreatureDamage()
	self:OnChatMsgSpellPeriodicHostileplayerDamage()
end

function TopScoreFu:OnChatMsgSpellPeriodicSelfBuffs()
	local target = "self"
	local amount, spell = babble.Deformat(arg1, PERIODICAURAHEALSELFSELF)
	if not amount then
		target, amount, spell = babble.Deformat(arg1, PERIODICAURAHEALSELFOTHER)
		if not amount then
			target = "self"
			amount, spell = babble.Deformat(arg1, PERIODICAURAHEALSELFSELF, RESIST_TRAILER)
			if not amount then
				target, amount, spell = babble.Deformat(arg1, PERIODICAURAHEALSELFOTHER, RESIST_TRAILER)
			end
		end
	end
	if amount then
		self:RecordHit(spell, tonumber(amount), "self", false, true)
	end
end

local _,class = UnitClass("player")
function TopScoreFu:DoesSpellExist(spell)
	if spell == babble.GetLocalizedSpell("ATTACK") then
		return true
	elseif class == "PALADIN" then
		if spell == babble.GetLocalizedSpell("JUDGEMENT_OF_LIGHT") then
			spell = babble.GetLocalizedSpell("SEAL_OF_LIGHT")
		elseif spell == babble.GetLocalizedSpell("JUDGEMENT_OF_RIGHTEOUSNESS") then
			spell = babble.GetLocalizedSpell("SEAL_OF_RIGHTEOUSNESS")
		elseif spell == babble.GetLocalizedSpell("JUDGEMENT_OF_WISDOM") then
			spell = babble.GetLocalizedSpell("SEAL_OF_WISDOM")
		elseif spell == babble.GetLocalizedSpell("JUDGEMENT_OF_JUSTICE") then
			spell = babble.GetLocalizedSpell("SEAL_OF_JUSTICE")
		elseif spell == babble.GetLocalizedSpell("JUDGEMENT_OF_COMMAND") then
			spell = babble.GetLocalizedSpell("SEAL_OF_COMMAND")
		elseif spell == babble.GetLocalizedSpell("JUDGEMENT_OF_THE_CRUSADER") then
			spell = babble.GetLocalizedSpell("SEAL_OF_THE_CRUSADER")
		end
	end
	local i = 1
	while true do
		local name = GetSpellName(i, "spell")
		if name == nil or name == "" then
			return false
		elseif name == spell then
			return true
		end
		i = i + 1
	end
end

function TopScoreFu:RecordHit(spell, amount, target, isCritical, isHeal)
	if UnitIsCharmed("player") then
		return
	end
	local alpha = string.find(spell, "%s*%(")
	if alpha ~= nil then
		spell = string.sub(spell, alpha - 1)
	end
	if self.charData.hits[spell] == false then
		return
	elseif self.charData.hits[spell] == nil then
		if not self:DoesSpellExist(spell) then
			self.charData.hits[spell] = false
			return
		end
		self.charData.hits[spell] = {
			crit = {},
			critPvP = {},
			normal = {},
			normalPvP = {},
			isHeal = isHeal,
		}
	end
	local t
	local tPvP
	if isCritical then
		t = self.charData.hits[spell].crit
		tPvP = self.charData.hits[spell].critPvP
	else
		t = self.charData.hits[spell].normal
		tPvP = self.charData.hits[spell].normalPvP
	end
	if t.amount == nil or t.amount < amount then
		local level
		local class
		local gameTarget
		if target == "self" then
			gameTarget = "player"
		elseif UnitName("target") == target then
			gameTarget = "target"
		elseif isHeal then
			if UnitExists("raid1") then
				for i = 1, GetNumRaidMembers() do
					local name = UnitName("raid" .. i)
					if name == nil then
						break
					elseif name == target then
						gameTarget = "raid" .. i
						break
					elseif UnitName("raidpet" .. i) == target then
						gameTarget = "raidpet" .. i
						break
					end
				end
			elseif UnitExists("party1") then
				for i = 1, GetNumPartyMembers() do
					local name = UnitName("party" .. i)
					if name == nil then
						break
					elseif name == target then
						gameTarget = "party" .. i
						break
					elseif UnitName("partypet" .. i) == target then
						gameTarget = "partypet" .. i
						break
					end
				end
			end
		end
		if gameTarget ~= nil then
			level = UnitLevel(gameTarget)
			_,class = UnitClass(gameTarget)
			if not self:IsShowingTrivial() and UnitIsTrivial(gameTarget) then
				return
			end
			t.amount = amount
			t.target = target
			t.level = level
			t.class = class
			if UnitIsPVP(gameTarget) and UnitPlayerControlled(gameTarget) then
				ace.CopyTable(tPvP, t)
			end
			if not self:IsFiltering(spell) and (not isHeal or self:IsIncludingHeals()) and (isHeal or self:IsIncludingDamage()) and (UnitIsPVP(gameTarget) or not self:IsOnlyPvP()) then
				if self:IsShowingSplash() then
					if isCritical then
						TopScoreFuSplash:AddMessage(format(self.loc.PATTERN_NEW_CRITICAL_RECORD, spell, amount), 1, 1, 0, 1, 3)
					else
						TopScoreFuSplash:AddMessage(format(self.loc.PATTERN_NEW_NORMAL_RECORD, spell, amount), 1, 1, 0, 1, 3)
					end
				end
				if self:IsPlayingNoise() then
					PlaySound("LEVELUP")
				end
				if self:IsTakingScreenshots() then
					TakeScreenshot()
				end
				self:Update()
			end
		end
	end
end

function TopScoreFu:UpdateData()
	self.highCrit = 0
	self.highNormal = 0
	for spell, t in self.charData.hits do
		if t and not self:IsFiltering(spell) and (not t.isHeal or self:IsIncludingHeals()) and (t.isHeal or self:IsIncludingDamage()) then
			local crit = t.crit
			local normal = t.normal
			if self:IsOnlyPvP() then
				crit = t.critPvP
				normal = t.normalPvP
			end
			if crit ~= nil and crit.amount ~= nil and self.highCrit < crit.amount then
				self.highCrit = crit.amount
			end
			if normal ~= nil and normal.amount ~= nil and self.highNormal < normal.amount then
				self.highNormal = normal.amount
			end
		end
	end
end

function TopScoreFu:UpdateText()
	self:SetText(format("|cffffffff%d|r/|cffffffff%d|r", self.highCrit, self.highNormal))
end

function TopScoreFu:UpdateTooltip()
	local addedHint = false
	for spell, t in self.charData.hits do
		if t and not self:IsFiltering(spell) and (not t.isHeal or self:IsIncludingHeals()) and (t.isHeal or self:IsIncludingDamage()) then
			local crit = t.crit
			local normal = t.normal
			if self:IsOnlyPvP() then
				crit = t.critPvP
				normal = t.normalPvP
			end
			if (crit and crit.amount) or (normal and normal.amount) then
				local cat = tablet:AddCategory(
					'text', spell,
					'columns', 2,
					'child_textR', 1,
					'child_textG', 1,
					'child_textB', 0
				)
				
				if crit ~= nil and crit.amount ~= nil then
					local color = "ffffff"
					if crit.amount == self.highCrit then
						color = "00ff00"
					end
					local target = crit.target
					if target == "self" then
						target = UnitName("player")
					end
					local r2, g2, b2 = babble.GetClassColor(crit.class)
					cat:AddLine(
						'text', self.loc.TEXT_CRITICAL .. " [|cff" .. color .. crit.amount .. "|r]",
						'text2', target .. " [|cffffffff" .. crit.level .. "|r]",
						'text2R', r2,
						'text2G', g2,
						'text2B', b2
					)
				end
				if normal ~= nil and normal.amount ~= nil then
					local color = "ffffff"
					if crit.amount == self.highCrit then
						color = "00ff00"
					end
					local target = normal.target
					if target == "self" then
						target = UnitName("player")
					end
					local r2, g2, b2 = babble.GetClassColor(crit.class)
					cat:AddLine(
						'text', self.loc.TEXT_NORMAL .. " [|cff" .. color .. normal.amount .. "|r]",
						'text2', target .. " [|cffffffff" .. normal.level .. "|r]",
						'text2R', r2,
						'text2G', g2,
						'text2B', b2
					)
				end
			end
			if not addedHint then
				addedHint = true
				tablet:SetHint(self.loc.HINT)
			end
		end
	end
end
	
function TopScoreFu:OnClick()
	if IsShiftKeyDown() and ChatFrameEditBox:IsVisible() then
		local critSpell, critAmount, normalSpell, normalAmount
		self:UpdateData()
		for spell, t in self.charData.hits do
			if t and not self:IsFiltering(t) and (not t.isHeal or self:IsIncludingHeals()) and (t.isHeal or self:IsIncludingDamage()) then
				local crit = t.crit
				local normal = t.normal
				if self:IsOnlyPvP() then
					crit = t.critPvP
					normal = t.normalPvP
				end
				if crit.amount == self.highCrit then
					critSpell = spell
					critAmount = crit.amount
				end
				if normal.amount == self.highNormal then
					normalSpell = spell
					normalAmount = normal.amount
				end
			end
		end
		local s = ""
		if normalSpell ~= nil then
			s = s .. format(self.loc.PATTERN_NORMAL_SPELL, normalSpell) .. ": " .. normalAmount
		end
		if critSpell ~= nil then
			if s ~= "" then
				s = s .. " || "
			end
			s = s .. format(self.loc.PATTERN_CRITICAL_SPELL, critSpell) .. ": " .. critAmount
		end
		ChatFrameEditBox:Insert(s)
	end
end

TopScoreFu:RegisterForLoad()